<?php

function get_theme_url($theme_name) {
    return get_base_url_with_slash().'/'.drupal_get_path('theme', $theme_name);
}

function is_active($theme_name) {
    $info = get_themes_info();
    return $info && isset($info['themes'][$theme_name]) && $info['themes'][$theme_name]['isActive'];
}

function get_themes_info() {
    $default_theme = \Drupal::config('node.settings')->get('theme_default');

    $base_url = get_base_url_with_slash();
    $themes = get_themler_themes();
    $result = array();
    foreach ($themes as $theme_name => $theme_dir) {
        $theme_url = get_theme_url($theme_name);
        $urls = get_urls($base_url, $theme_name);
        $url = $urls['themler'];
        $result['themes'][$theme_name] = array(
            'themeName' => $theme_name,
            'thumbnailUrl' => $theme_url. '/versions/8.x/screenshot.png',
            'openUrl' => $url . '?theme=' . $theme_name . '&ver=' . theme_get_manifest_version($theme_name),
            'isActive' => $default_theme && ($default_theme === $theme_name),
        );
    }
    return $result;
}

function list_themes() {
    return \Drupal::service('theme_handler')->listInfo();  // Get a list of available themes.
}

function get_themler_themes() {
    $themes = list_themes(true);
    $themler_themes = array();
    foreach ($themes as $theme_name => $theme) {
        $theme_dir = get_theme_dir($theme_name);
        $project = get_project($theme_dir);
        if (count((array)$project)) {
            $themler_themes[$theme_name] = $theme_dir;
        }
    }
    return $themler_themes;
}

function remove_theme($theme_name) {
    if (is_active($theme_name))
        return array('errors' => 'Theme `' . $theme_name .'` is active');

    ProviderLog::start('remove_theme');

    $theme_dir = get_theme_dir($theme_name);
    $dirs = array($theme_name => $theme_dir, get_preview_theme_name($theme_name) => get_preview_theme_dir($theme_dir));
    foreach ($dirs as $name => $dir) {
        if (!is_dir($dir))
            return array('errors' => 'Theme `' . $name .'` does not exist');
        FilesHelper::remove_dir($dir);
    }

    ProviderLog::start('drupal_flush_all_caches');
    drupal_flush_all_caches();
    ProviderLog::end('drupal_flush_all_caches');
    ProviderLog::end('remove_theme');
    return array('result' => 'done', 'log' => ProviderLog::getLog());
}

function activate_theme($theme_name, $new_theme_name = '') {
    $theme  = $new_theme_name ? $new_theme_name : $theme_name;
    $result = array('error' => 'Theme `'. $theme .'` does not exist');

    $config = \Drupal::configFactory()->getEditable('system.theme');
    $themes = \Drupal::state()->get('system.theme.data', array());

    if (isset($themes[$theme])) {
        $config->set('default', $theme)->save();
        $result = array('result' => 'done');
    }

    return $result;
}

function copy_theme($theme_name, $new_theme_name = '') {
    ProviderLog::start('copy_theme');
    $new_theme_name = check_theme_name($new_theme_name ? $new_theme_name : $theme_name);
    $renamed_data = get_renamed_theme_data($theme_name, $new_theme_name);
    check_preview($theme_name); // inner preview can exist if theme was upload from Themler in zip archive
    process_copy_rename($renamed_data, $theme_name, $new_theme_name);
    ProviderLog::end('copy_theme');

    return array('result' => 'done', 'log' => ProviderLog::getLog());
}

function rename_theme($theme_name, $new_theme_name) {
    $rename = can_rename($theme_name, $new_theme_name);
    if ($rename['message'])
        return $rename;

    ProviderLog::start('rename_theme');
    $renamed_data = get_renamed_theme_data($theme_name, $new_theme_name);
    check_preview($theme_name); // inner preview can exist if theme was upload from Themler in zip archive
    process_copy_rename($renamed_data, $theme_name, $new_theme_name, true);
    ProviderLog::end('rename_theme');

    return array('result' => 'done', 'log' => ProviderLog::getLog());
}

function can_rename($theme_name, $new_theme_name) {
    $result = theme_name_validate($new_theme_name);
    if ($result['message'])
        return $result;

    $msg = '';
    $renamedData = get_renamed_theme_data($theme_name, $new_theme_name);
    if (!isset($renamedData))
        $msg = 'Theme name is invalid';

    $path = '';//TODO: check error when new theme not exists in drupal_get_path('theme', $renamedData['new_theme_name']);
    if (!empty($path) || file_exists($renamedData['new_theme_dir']))
        $msg = 'Theme with such name already exists';

    return array('result' => 'done', 'message' => $msg);
}

function get_renamed_theme_data($theme_name, $new_theme_name = '') {
    $theme_dir = get_theme_dir($theme_name);
    return get_theme_data($theme_name, $theme_dir, $theme_name, $new_theme_name);
}

function get_uploaded_theme_data($theme_name, $uploaded_themes_dir, $uploaded_theme_name, $new_theme_name = '') {
    $uploaded_theme_dir = $uploaded_themes_dir . '/' . $uploaded_theme_name;
    return get_theme_data($theme_name, $uploaded_theme_dir, $uploaded_theme_name, $new_theme_name);
}

function get_theme_data($theme_name, $uploaded_theme_dir, $uploaded_theme_name, $new_theme_name) {
    $theme_dir = get_theme_dir($theme_name);
    $result = array(
                    'theme_dir' => $uploaded_theme_dir,
                    'theme_name' => $uploaded_theme_name,
                    'preview_theme_name' => get_preview_theme_name($uploaded_theme_name),
                    'preview_theme_dir' => get_preview_theme_dir($uploaded_theme_dir)
                );

    if ($new_theme_name && theme_name_is_valid($new_theme_name)) {
        $new_theme_dir = str_replace($theme_name, $new_theme_name, $theme_dir);
        $result = array_merge($result, 
                    array('new_theme_name' => $new_theme_name,
                          'new_theme_dir' => $new_theme_dir,
                          'new_preview_dir' => get_preview_theme_dir($new_theme_dir),
                          'new_preview_name' => get_preview_theme_name($new_theme_name)
                    ));
    }

    return $result;
}

function theme_name_validate($theme_name) {
    $msg = '';
    if (empty($theme_name)) 
        $msg = 'Theme name cannot be empty';
    if (!theme_name_is_valid($theme_name))
        $msg = 'Theme name cannot start with numbers';
    return array('result' => 'done', 'message' => $msg);
}

function theme_name_is_valid($new_theme_name) {
    return preg_match('|^[A-Za-z]([A-Za-z\d_]+)|i', $new_theme_name) == 1;
}

function process_copy_rename($renamed_data, $theme_name, $new_theme_name, $rename = false) {
    if (file_exists($renamed_data['preview_theme_dir'])) {
        $replace_data = array('method' => array('FilesHelper', 'replace_theme_name'), 'old_name' => $renamed_data['preview_theme_name'], 'new_name' => $renamed_data['new_preview_name']);
        FilesHelper::copy_recursive($renamed_data['preview_theme_dir'], $renamed_data['new_preview_dir'], null, $replace_data);
    }
    
    $replace_data = array('method' => array('FilesHelper', 'replace_theme_name'), 'old_name' => $theme_name, 'new_name' => $new_theme_name);
    FilesHelper::copy_recursive($renamed_data['theme_dir'], $renamed_data['new_theme_dir'], null, $replace_data);
    
    if ($rename) {
        FilesHelper::remove_dir($renamed_data['theme_dir']);
        FilesHelper::remove_dir($renamed_data['preview_theme_dir']);
    }
    themler_enable_theme($new_theme_name);
}

function check_preview($theme_name) {
    ProviderLog::start('check_preview');
    $theme_dir = get_theme_dir($theme_name);
    $preview_theme_name = $theme_name . PREVIEW_SUFFIX;
    $inner_preview_dir = $theme_dir . '/' . $preview_theme_name;
    $preview_dir = get_preview_theme_dir($theme_dir);
    move_preview($preview_dir, $inner_preview_dir);
    ProviderLog::end('check_preview');
}

function move_preview($preview_dir, $inner_preview_dir) {
    if (file_exists($inner_preview_dir)) {
        if (file_exists($preview_dir))
            FilesHelper::empty_dir($preview_dir, true);
        FilesHelper::rename_file($inner_preview_dir, $preview_dir);
    }
}

function check_theme_name($theme_name) {
    $themes = list_themes(true);

    while (array_key_exists($theme_name, $themes)) {
        preg_match('#(.*?)(\d*)$#', $theme_name, $m);
        $theme_name = $m[1];
        $suffix = (int) $m[2];
        $suffix++;
        $theme_name .= $suffix;
    };
    
    return $theme_name;
}

function upload($theme_name, $callback) {
    $filename = isset($_REQUEST['filename']) ? $_REQUEST['filename'] : false;
    $is_last = isset($_REQUEST['last']) ? $_REQUEST['last'] : false;
    $content_range = $_SERVER['HTTP_CONTENT_RANGE'];

    $base_upload_dir = get_themler_storage_dir();

    if (empty($base_upload_dir)) {
        $result = array(
            'status' => 'error',
            'message' => 'Upload folder error: ' . $base_upload_dir
        );
    } elseif (!isset($_FILES['chunk']) || !file_exists($_FILES['chunk']['tmp_name'])) {
        $result = array(
            'status' => 'error',
            'message' => 'Empty chunk data'
        );
    } elseif (!$content_range && !$is_last) {
        $result = array(
            'status' => 'error',
            'message' => 'Empty Content-Range header'
        );
    } elseif (!$filename) {
        $result = array(
            'status' => 'error',
            'message' => 'Empty file name'
        );
    } else {
        $tmp_path = $base_upload_dir . '/' . $filename . '.tmp';
        $range_begin = 0;

        if ($content_range) {
            list($range, $total) = explode('/', str_replace('bytes ', '', $content_range));
            list($range_begin, $range_end) = explode('-', $range);
        }

        $file = fopen($tmp_path, 'c');

        if (flock($file, LOCK_EX)) {

            fseek($file, (int) $range_begin);
            fwrite($file, FilesHelper::read_file($_FILES['chunk']['tmp_name']));

            flock($file, LOCK_UN);
            fclose($file);
        }

        if ($is_last) {
            return call_user_func($callback, $theme_name, $filename, $tmp_path);
        } else {
            $result = array(
                'status' => 'processed'
            );
        }
    }

    return $result;
}

function unzip($theme_name, $filename, $tmp_path) {
    $base_upload_dir = get_themler_storage_dir();
    $archive_file = $base_upload_dir . '/' . $filename .'.tmp';
    $tmp_dir = $base_upload_dir . '/zip-data.tmp';
    FilesHelper::empty_dir($tmp_dir);

    $archive = new PclZip($archive_file);

    if (0 == $archive->extract(PCLZIP_OPT_PATH, $tmp_dir)) {
        return array(
            'status' => 'error',
            'message' => "<p><b>Invalid zip</b></p>Extract error : " . $archive->errorInfo(true)
        );
    }

    $fso = FilesHelper::generate_fso($tmp_dir);
    FilesHelper::delete_file($archive_file);
    FilesHelper::empty_dir($tmp_dir);
    return array('status' => 'done', 'fso' => $fso);
}

function upload_image($theme_name, $filename, $tmp_path) {
    $data = get_renamed_theme_data($theme_name);
    $images_dir = $data['preview_theme_dir'] . '/images';
    $base_path = $images_dir . '/' . $filename;

    if (file_exists($base_path))
        FilesHelper::delete_file($base_path);
    FilesHelper::create_dir(dirname($base_path), 0776, true);
    FilesHelper::rename_file($tmp_path, $base_path);

    $changed_files = get_preview_changed_files($data['theme_dir']);
    $changed_files[] = $base_path;
    set_preview_changed_files($data['theme_dir'], $data['preview_theme_dir'], $changed_files);
    return array(
        'status' => 'done',
        'url' => $GLOBALS['base_url'] . '/' . drupal_get_path('theme', $theme_name . '_preview') . '/images/' . $filename
    );
}

function upload_theme($theme_name, $filename, $tmp_path) {
    $uniqid = uniqid();
    $base_upload_dir = get_themler_storage_dir();
    $sandbox = $base_upload_dir . DIRECTORY_SEPARATOR . $uniqid . DIRECTORY_SEPARATOR;
    FilesHelper::create_dir($sandbox);
    $result = extract_theme($tmp_path, $sandbox);
    if (array_key_exists('error', $result))
        return $result;

    $uploaded_themes_dir = $sandbox . 'themes';
    $themes = scandir($uploaded_themes_dir);
    $archive_theme_name = '';
    foreach($themes as $value)
        if (preg_match('#(.*)_preview$#', $value, $matches)) {
            $archive_theme_name = $matches[1];
        }

    if ($archive_theme_name) {
        $new_name = check_theme_name($archive_theme_name);
        $uploaded_data = get_uploaded_theme_data($theme_name, $uploaded_themes_dir, $archive_theme_name, $new_name);
        move_preview($uploaded_data['preview_theme_dir'], $uploaded_data['theme_dir'].'/'.$archive_theme_name.PREVIEW_SUFFIX);
        process_copy_rename($uploaded_data, $archive_theme_name, $new_name);
        FilesHelper::empty_dir($sandbox, true);
        $result = array(
            'status' => 'done',
            'path' => $tmp_path,
            'new_theme_name' => $new_name
        );
    } else {
        $result = array(
            'status' => 'error',
            'errors' => 'Please make sure, that you have selected a valid Drupal theme.'
        );
    }

    return $result;
}

function extract_theme($theme_zip_file, $dir)  {
    $status = false;
    if ($status = zip_extract($theme_zip_file, $dir)) {
        foreach (FilesHelper::enumerate_files($dir, false) as $file) {
            $path = $file['path'];
            $ext  = pathinfo($path, PATHINFO_EXTENSION);
            if (in_array($ext, array('zip')))
                $status &= zip_extract($path, $dir . 'themes');
        }
    }

    if ($status)
        return array('status' => 'done');
    return array('status' => 'error', 'message' => 'Error during zip extraction');
}